#ifndef HARKIO_HARK_HARKFILEREADER_HPP_
#define HARKIO_HARK_HARKFILEREADER_HPP_

#include <boost/shared_ptr.hpp>
#include <harkio2/AnalyzeError.hpp>

/** @file HarkFileReader.hpp
 * @brief HarkFileReaderクラスの定義
 */

namespace harkio2
{
namespace hark
{

class HarkFile;

/** @brief HARKファイルの読み込みインターフェース。
 *
 * このインターフェースを実装した、各種ファイル形式（例えばバイナリ形式）
 * に対応したクラスを用いることにより、様々なフォーマットのHARKファイルからデータを読み込むことができる。
 *
 * @sa HarkFileBinaryReader
 */
class HarkFileReader
{
public:
    /// @brief 継承のための仮想デストラクタ。
    virtual ~HarkFileReader() {}

    /** @brief HARKファイルからデータを読み込んで返す。
     *
     * HARKファイルのデータをどこかから読み込み、返す。
     * 読み込む場所はサブクラスのコンストラクタやメソッドにより指定されるだろう。
     *
     * このインターフェースでは、HARKファイルが本当のファイルであるかどうかは規定していないため、
     * 例えば名前付きパイプなどから読み込むようなサブクラスを定義することもできる。
     * @return 読み込んだデータ
     */
    virtual boost::shared_ptr<harkio2::hark::HarkFile> Read() throw (harkio2::AnalyzeError) = 0;
};

}
}

#endif // HARKIO_HARK_HARKFILEREADER_HPP_

