#define BOOST_TEST_MODULE "testcase for NoiseSource class and NoiseSource1 class"
#define BOOST_TEST_DYN_LINK
#include "common.hpp"

#include <boost/test/unit_test.hpp>

#include <string>
#include <boost/shared_ptr.hpp>
#include <harkio2/harkconfig/NoiseSource.hpp>
#include "rangecheck.hpp"
using namespace boost;
using namespace std;

typedef boost::shared_ptr<harkio2::harkconfig::NoiseSource> nsp;
typedef boost::shared_ptr<harkio2::harkconfig::NoiseSource1> nsp1;

BOOST_AUTO_TEST_CASE(versioning_test)
{
    nsp ns0(new harkio2::harkconfig::NoiseSource1());
    BOOST_CHECK_EQUAL(ns0->GetVersion(), 1);
}

BOOST_AUTO_TEST_CASE(getter_setter_test)
{
    nsp ns0(new harkio2::harkconfig::NoiseSource1());

    BOOST_CHECK_EQUAL(ns0->GetVersion(), 1);

    nsp1 ns1 = dynamic_pointer_cast<harkio2::harkconfig::NoiseSource1>(ns0);

    BOOST_CHECK(ns1 != 0);

    if (ns1 != 0)
    {
        BOOST_CHECK_EQUAL(ns1->GetVersion(), 1);

        BOOST_CHECK_THROW(ns1->GetNoiseIndex(), harkio2::NotInitialized);
        BOOST_CHECK_THROW(ns1->GetComment(), harkio2::NotInitialized);

        ns1->SetNoiseIndex(3);
        ns1->SetPosition(harkio2::MakeDirectionFromVector<float>(harkio2::Vector3<float>(1, 2, 3)));
        ns1->SetComment(u8"ノイズ情報");

        BOOST_CHECK_NO_THROW(ns1->GetNoiseIndex());
        BOOST_CHECK_NO_THROW(ns1->GetComment());
        BOOST_CHECK_EQUAL(ns1->GetNoiseIndex(), 3);
        BOOST_CHECK(rangecheck::CheckRange<float>(ns1->GetPosition().GetUnitVector(), harkio2::Normalize(harkio2::Vector3<float>(1, 2, 3)), 0.01));

    }
}

